<?php

namespace App\Http\Controllers;

use App\Models\User;
use App\Models\Jabatan;
use App\Models\Pangkat;
use App\Models\Agama;
use App\Models\RiwayatPegawai;
use Illuminate\Http\Request;
use DataTables;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Storage;

class PegawaiController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $data = User::pegawai()->leftJoin('riwayat_pegawai', function($q){
                $q->on('riwayat_pegawai.user_id', '=', 'users.id')->where('riwayat_pegawai.status', 1);
            })->leftJoin('jabatans', function($q){
                $q = $q->on('jabatans.id', '=', 'riwayat_pegawai.jabatan_id'); 
            });
            if($request['keyword'] !== null){
                $data = $data->where('users.name', 'LIKE', "%".$request['keyword']."%")->orWhere('users.no_pegawai', 'LIKE', "%".$request['keyword']."%");
            }
            if($request['jabatan_id'] !== null){
                $data = $data->where('riwayat_pegawai.jabatan_id', $request['jabatan_id']);
            }
            if($request['pangkat_id'] !== null){
                $data = $data->where('riwayat_pegawai.pangkat_id', $request['pangkat_id']);
            }
            if($request['jenis_pegawai'] !== null){
                $data = $data->where('users.pegawai_type', $request['jenis_pegawai']);
            }
            $data = $data->orderBy('users.created_at', 'desc')
            ->select(
                'users.id', 
                'users.photo', 
                'users.name', 
                'users.no_pegawai', 
                'users.email', 
                'users.phone', 
                'users.status', 
                'jabatans.name as jabatan_name')
            ->get();
            return Datatables::of($data)
                    ->addIndexColumn()
                    ->addColumn('jabatan', function($row){
                        $jabatan = 'Jabatan belum diatur';
                        if($row->jabatan_name !== null){
                            $jabatan =  $row->jabatan_name;
                        }
                        return $jabatan;
                    })
                    ->addColumn('status', function($row){
                        return config('constants.status_pegawai.'.$row->status);
                    })
                    ->addColumn('photo', function($row){
                        if($row->photo){
                            $url= asset("uploads/".$row->photo);
                            $photo = '<img class="c-avatar-img" src="'.$url.'" alt="Image" style="width:100%;"/>';
                        } else {
                            $photo = '-';
                        }
                        return $photo;
                    })
                    ->addColumn('action', function($row){
     
                        $btn = '<div class="d-flex justify-content-center">'.
                           '<a href="/pegawai/'.$row->id.'" class="btn btn-sm btn-info mr-2"><span class="cil-user mr-1"></span> Detail</a>'.
                           '<a href="/pegawai/'.$row->id.'/edit" class="btn btn-sm btn-success mr-2"><span class="cil-pencil mr-1"></span> Edit</a>'.
                            '</div>';
                        return $btn;
                    })
                    ->rawColumns(['jabatan', 'photo', 'action', 'status'])
                    ->make(true);
        }
        $jabatans = Jabatan::pluck('name','id');
        $pangkats = Pangkat::pluck('name','id');
        return view('dashboard.pegawai.index', [
            'jabatans' => $jabatans,
            'pangkats' => $pangkats
        ]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $jabatans = Jabatan::pluck('name', 'id');
        $pangkats = Pangkat::pluck('name', 'id');
        $religions = Agama::pluck('name', 'id');
        return view('dashboard.pegawai.create', [
            'jabatans' => $jabatans,
            'pangkats' => $pangkats,
            'religions' => $religions
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validate = $request->validate([
            'name' => 'required',
            'no_pegawai' => 'required',
            'email' => 'required|unique:users,email',
            'phone' => 'required',
            'image' => 'image|mimes:jpeg,png,jpg|max:2048',
            'jabatan_id' => 'required',
            'pangkat_id' => 'required'
        ]);

        try {
            //save pegawai
            $pegawai = new User();
            $pegawai->name = $request['name'];
            $pegawai->username = str_replace(' ', '', $request['no_pegawai']);
            $pegawai->email = $request['email'];
            $pegawai->password = Hash::make('12345678');
            $pegawai->menuroles = 'pegawai';
            $pegawai->no_pegawai = $request['no_pegawai'];
            $pegawai->pegawai_type = $request['pegawai_type'];
            $pegawai->gender = $request['gender'];
            $pegawai->birth_place = $request['birth_place'];
            $pegawai->birth_date = $request['birth_date'];
            $pegawai->phone = $request['phone'];
            $pegawai->address = $request['address'];
            $pegawai->religion_id = $request['religion_id'];
            $pegawai->npwp = $request['npwp'];
            $pegawai->status = $request['status'];
            $pegawai->gaji_berkala = isset($request['maksimal']) ? null : $request['gaji_berkala'];
            $pegawai->maksimal = isset($request['maksimal'])?1:0;
            if ($request->file('photo')) {
                $photo = $request->file('photo')->store('pegawai', ['disk' => 'public']);
                $pegawai->photo = $photo;
            }
            $pegawai->account_bank = $request['account_bank'];
            $pegawai->account_no = $request['account_no'];
            $pegawai->save();

            // assign role
            $pegawai->assignRole('pegawai');
            
            // save riwayat pegawai
            $riwayat = new RiwayatPegawai();
            $riwayat->user_id = $pegawai->id;
            $riwayat->jabatan_id = $request['jabatan_id'];
            $riwayat->pangkat_id = $request['pangkat_id'];
            $riwayat->tmt_jabatan = $request['tmt_jabatan'];
            $riwayat->tmt_pangkat = $request['tmt_pangkat'];
            $riwayat->gaji_pokok = $request['gaji_pokok'];
            $riwayat->tunjangan = $request['tunjangan'];
            $riwayat->uang_makan = $request['uang_makan'];
            $riwayat->uang_transportasi = $request['uang_transportasi'];
            $riwayat->save();
            return redirect()->route('pegawai.index')->with('success', 'Pegawai '.$pegawai->name.' berhasil disimpan');
        } catch (\Throwable $th) {
            return redirect()->back()->with('fail', $th->getMessage())->withInput();
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Pegawai  $pegawai
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $pegawai = User::where('id', $id)->with('riwayat.pangkat', 'riwayat.jabatan', 'agama')->first();
        $riwayatAktif = $pegawai->riwayat()->where('status', 1)->first();
        return view('dashboard.pegawai.show', [
            'pegawai' => $pegawai,
            'riwayat_aktif' => $riwayatAktif
        ]);
    }

    public function detailPegawai()
    {
        $pegawai = User::where('id', auth()->user()->id)->with('riwayat.pangkat', 'riwayat.jabatan', 'agama')->first();
        $riwayatAktif = $pegawai->riwayat()->where('status', 1)->first();
        return view('dashboard.pegawai.show', [
            'pegawai' => $pegawai,
            'riwayat_aktif' => $riwayatAktif
        ]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Pegawai  $pegawai
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $pegawai = User::find($id);
        $jabatans = Jabatan::pluck('name', 'id');
        $pangkats = Pangkat::pluck('name', 'id');
        $religions = Agama::pluck('name', 'id');
        return view('dashboard.pegawai.edit', [
            'jabatans' => $jabatans,
            'pangkats' => $pangkats,
            'religions' => $religions,
            'pegawai' => $pegawai
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Pegawai  $pegawai
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $validate = $request->validate([
            'name' => 'required',
            'no_pegawai' => 'required',
            'email' => 'required|unique:users,email,'.$id,
            'phone' => 'required',
            'image' => 'image|mimes:jpeg,png,jpg|max:2048',
        ]);

        $pegawai = User::find($id);
        $pegawai->name = $request['name'];
        $pegawai->username = str_replace(' ', '', $request['no_pegawai']);
        $pegawai->email = $request['email'];
        $pegawai->no_pegawai = $request['no_pegawai'];
        $pegawai->pegawai_type = $request['pegawai_type'];
        $pegawai->gender = $request['gender'];
        $pegawai->birth_place = $request['birth_place'];
        $pegawai->birth_date = $request['birth_date'];
        $pegawai->phone = $request['phone'];
        $pegawai->address = $request['address'];
        $pegawai->religion_id = $request['religion_id'];
        $pegawai->account_no = $request['account_no'];
        $pegawai->account_bank = $request['account_bank'];
        $pegawai->npwp = $request['npwp'];
        $pegawai->status = $request['status'];
        $pegawai->gaji_berkala = isset($request['maksimal']) ? null : $request['gaji_berkala'];
        $pegawai->maksimal = isset($request['maksimal'])?1:0;
        if ($request->file('photo')) {
            $photo = $request->file('photo')->store('pegawai', ['disk' => 'public']);
            $pegawai->photo = $photo;
        }
        $pegawai->save();

        if(auth()->user()->menuroles != 'pegawai') {
            return redirect()->route('pegawai.index')->with('success', 'Pegawai '.$pegawai->name.' berhasil diubah');
        } else {
            return redirect()->route('pegawai.detail-pegawai')->with('success', 'Data anda berhasil diubah');
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Pegawai  $pegawai
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    public function createRiwayat($id)
    {
        $jabatans = Jabatan::pluck('name', 'id');
        $pangkats = Pangkat::pluck('name', 'id');
        return view('dashboard.pegawai.riwayat.create', [
            'id' => $id,
            'jabatans' => $jabatans,
            'pangkats' => $pangkats
        ]);
    }
    
    public function storeRiwayat(Request $request, $id)
    {
        $riwayat = new RiwayatPegawai();
        $riwayat->user_id = $id;
        $riwayat->jabatan_id = $request['jabatan_id'];
        $riwayat->pangkat_id = $request['pangkat_id'];
        $riwayat->tmt_jabatan = $request['tmt_jabatan'];
        $riwayat->tmt_pangkat = $request['tmt_pangkat'];
        $riwayat->gaji_pokok = $request['gaji_pokok'];
        $riwayat->tunjangan = $request['tunjangan'];
        $riwayat->uang_makan = $request['uang_makan'];
        $riwayat->uang_transportasi = $request['uang_transportasi'];
        $riwayat->status = $request['status'];
        $riwayat->save();

        if($request['status'] == 1){
            $update = RiwayatPegawai::where('user_id', $id)->where('id', '!=', $riwayat->id)->update(['status' => 2]);
        }

        return redirect()->route('pegawai.show', $id)->with('success', 'Riwayat pegawai berhasil disimpan');
    }

    public function editRiwayat($id)
    {
        $riwayat = RiwayatPegawai::find($id);
        $jabatans = Jabatan::pluck('name', 'id');
        $pangkats = Pangkat::pluck('name', 'id');
        return view('dashboard.pegawai.riwayat.edit', [
            'riwayat' => $riwayat,
            'jabatans' => $jabatans,
            'pangkats' => $pangkats
        ]);
    }

    public function updateRiwayat(Request $request, $id)
    {
        $request->validate([
            'jabatan_id' => 'required',
            'pangkat_id' => 'required',
        ]);
        
        $riwayat = RiwayatPegawai::find($id);
        if($request['status'] == 1){
            $status = RiwayatPegawai::where('user_id', $riwayat->user_id)->where('id', '!=', $id)->update(['status' => 2]);
        }
        else if($request['status'] == 2){
            $status = RiwayatPegawai::where('user_id', $riwayat->user_id)->where('status', 1)->count();
            if($status == 1){
                return redirect()->back()->withInput()->with('error', 'Pegawai harus memiliki satu riwayat aktif');
            }
        }
        $riwayat->jabatan_id = $request['jabatan_id'];
        $riwayat->pangkat_id = $request['pangkat_id'];
        $riwayat->tmt_jabatan = $request['tmt_jabatan'];
        $riwayat->tmt_pangkat = $request['tmt_pangkat'];
        $riwayat->gaji_pokok = $request['gaji_pokok'];
        $riwayat->tunjangan = $request['tunjangan'];
        $riwayat->uang_makan = $request['uang_makan'];
        $riwayat->uang_transportasi = $request['uang_transportasi'];
        $riwayat->status = $request['status'];
        $riwayat->save();


        return redirect()->route('pegawai.show', $riwayat->user_id)->with('success', 'Riwayat pegawai berhasil diubah');
    }
}
