<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Models\Permission;
use Illuminate\Support\Facades\DB;
use App\Models\Menurole;
use App\Models\RoleHierarchy;
use DataTables;

class RolesController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if($request->ajax()){
            $roles = Role::with('permissions')
            ->leftJoin('role_hierarchy', 'roles.id', '=', 'role_hierarchy.role_id')
            ->select('roles.*', 'role_hierarchy.hierarchy')
            ->orderBy('hierarchy', 'asc')
            ->get();

            return DataTables::of($roles)
            ->addIndexColumn()
            ->addColumn('permissions', function($row){
                $permissions = '';
                if($row->permissions != null){
                    foreach ($row->permissions as $permission) {
                        $permissions = $permissions.'<span class="badge badge-pill badge-secondary m-1">'.$permission->name.'</span>';
                    }
                }
                return $permissions;
            })
            ->addColumn('action', function($row){
                $btn = '<div class="d-flex justify-content-start">'.
                '<a href="'.route('roles.show', $row->id ).'" class="btn btn-sm btn-info mr-2"><span class="cil-user mr-1"></span> Detail</a>'.
                '<a href="'.route('roles.edit', $row->id ).'" class="btn btn-sm btn-success mr-2"><span class="cil-pencil mr-1"></span> Edit</a>';
                '<form class="form-delete" action="'.route('roles.destroy', $row->id ).'" method="POST">'.
                    '<input type="hidden" name="_method" value="delete" />'.
                    '<input type="hidden" name="_token" id="csrf-token" value="'.csrf_token().'" />'.
                    '<button class="btn btn-sm btn-danger"><span class="cil-trash mr-1"></span> Hapus</button>'.
                '</form>';
                return $btn;
             })
             ->rawColumns(['permissions', 'action'])
             ->make(true);
        }
        return view('dashboard.roles.index');
    }

    public function moveUp(Request $request){
        $element = RoleHierarchy::where('role_id', '=', $request->input('id'))->first();
        $switchElement = RoleHierarchy::where('hierarchy', '<', $element->hierarchy)
            ->orderBy('hierarchy', 'desc')->first();
        if(!empty($switchElement)){
            $temp = $element->hierarchy;
            $element->hierarchy = $switchElement->hierarchy;
            $switchElement->hierarchy = $temp;
            $element->save();
            $switchElement->save();
        }
        return redirect()->route('roles.index');
    }

    public function moveDown(Request $request){
        $element = RoleHierarchy::where('role_id', '=', $request->input('id'))->first();
        $switchElement = RoleHierarchy::where('hierarchy', '>', $element->hierarchy)
            ->orderBy('hierarchy', 'asc')->first();
        if(!empty($switchElement)){
            $temp = $element->hierarchy;
            $element->hierarchy = $switchElement->hierarchy;
            $switchElement->hierarchy = $temp;
            $element->save();
            $switchElement->save();
        }
        return redirect()->route('roles.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $permissions = Permission::pluck('name', 'id');
        return view('dashboard.roles.create', [
            'permissions' => $permissions
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $role = new Role();
        $role->name = $request->input('name');
        $role->save();
        $hierarchy = RoleHierarchy::select('hierarchy')
        ->orderBy('hierarchy', 'desc')->first();
        if(empty($hierarchy)){
            $hierarchy = 0;
        }else{
            $hierarchy = $hierarchy['hierarchy'];
        }
        $hierarchy = ((integer)$hierarchy) + 1;
        $roleHierarchy = new RoleHierarchy();
        $roleHierarchy->role_id = $role->id;
        $roleHierarchy->hierarchy = $hierarchy;
        $roleHierarchy->save();

        // save permissions
        $role->givePermissionTo($request['permissions']);

        $request->session()->flash('message', 'Successfully created role');
        return redirect()->route('roles.create');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $role = Role::where('id', $id)->with('permissions')->first();
        return view('dashboard.roles.show', array(
            'role' => $role
        ));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $role = Role::with('permissions')->where('id', '=', $id)->first();
        $permissions = Permission::pluck('name', 'id');

        return view('dashboard.roles.edit', array(
            'role' => $role,
            'permissions' => $permissions
        ));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $role = Role::where('id', '=', $id)->first();
        $role->name = $request->input('name');
        $role->save();

        // sync permissions
        $role->syncPermissions($request['permissions']);

        $request->session()->flash('message', 'Successfully updated role');
        return redirect()->route('roles.edit', $id); 
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id, Request $request)
    {
        $role = Role::where('id', '=', $id)->first();
        $roleHierarchy = RoleHierarchy::where('role_id', '=', $id)->first();
        $menuRole = Menurole::where('role_name', '=', $role->name)->first();
        if(!empty($menuRole)){
            $request->session()->flash('message', "Can't delete. Role has assigned one or more menu elements.");
            $request->session()->flash('back', 'roles.index');
            return view('dashboard.shared.universal-info');
        }else{
            $role->delete();
            $roleHierarchy->delete();
            $request->session()->flash('message', "Successfully deleted role");
            $request->session()->flash('back', 'roles.index');
            return view('dashboard.shared.universal-info');
        }
    }
}
