<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\User;
use App\Models\Instansi;
use Spatie\Permission\Models\Role;
use DataTables;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\DB;

class UsersController extends Controller
{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware('admin');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if($request->ajax()){
            $you = auth()->user();
            $users = User::select(
                'id', 'name', 'email', 'menuroles'
            )->get();

            return DataTables::of($users)
            ->addIndexColumn()
            ->addColumn('action', function($row)use($you){
    
                $btn = '<div class="d-flex justify-content-start">'.
                '<a href="'.route('users.show', $row->id ).'" class="btn btn-sm btn-info mr-2"><span class="cil-user mr-1"></span> Detail</a>'.
                '<a href="'.route('users.edit', $row->id ).'" class="btn btn-sm btn-success mr-2"><span class="cil-pencil mr-1"></span> Edit</a>';
                if($row->id !== $you->id){
                    $btn = $btn . '<form class="form-delete" action="'.route('users.destroy', $row->id ).'" method="POST">'.
                    '<input type="hidden" name="_method" value="delete" />'.
                    '<input type="hidden" name="_token" id="csrf-token" value="'.csrf_token().'" />'.
                    '<button class="btn btn-sm btn-danger"><span class="cil-trash mr-1"></span> Hapus</button>'.
                    '</form>';
                }
                return $btn;
             })
             ->rawColumns(['action'])
             ->make(true);
        }
        return view('dashboard.admin.usersList');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $user = User::with('instansi', 'roles')->where('id', $id)->first();
        return view('dashboard.admin.userShow', compact( 'user' ));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = User::find($id);
        $roles = Role::get();
        $instansis = Instansi::get();
        return view('dashboard.admin.userEditForm', compact('user', 'roles', 'instansis'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $validatedData = $request->validate([
            'name'      => 'required|min:1|max:256',
            'email'     => 'required|email|max:256',
            'password'  => 'nullable|min:8|confirmed'
        ]);
        
        try {
            DB::beginTransaction();
            $user = User::find($id);
            $user->name         = $request['name'];
            $user->username     = $request['email'];
            $user->instansi_id  = $request['instansi_id'];
            $user->email        = $request['email'];
            if($request['role']){
                $role = Role::find($request['role']);
                $user->menuroles    = $role->name;
                $user->syncRoles($role->name);
            }
            if($request['password'] != null){
                $user->password = Hash::make($request['password']);
            }
            $user->save();
            DB::commit();
            return redirect()->route('users.index')->with('success', 'Successfully updated user : '.$user->name);
        } catch (\Throwable $th) {
            DB::rollback();
            return redirect()->back()->with('fail', $th->getMessage())->withInput();
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $user = User::find($id);
        $name = $user->name;
        try {
            DB::beginTransaction();
            if($user){
                $user->removeRole($user->roles->pluck('name')->toArray());
                $user->delete();
            }
            DB::commit();
            return redirect()->route('users.index')->with('success', 'User '.$name.' has been deleted');
        } catch (\Throwable $th) {
            DB::rollback();
            return redirect()->back()->with('error', $th->getMessage());
        }
    }

    public function create()
    {
        $roles = Role::get();
        $instansis = Instansi::get();
        return view('dashboard.admin.userCreateForm', compact('roles', 'instansis'));
    }

    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|max:256',
            'role' => 'required',
            'email' => 'required|email|max:256|unique:users,email',
            'password' => 'required|confirmed|min:8',
            'password_confirmation' => 'required'
        ]);

        $role = Role::find($request['role']);
        try {
            DB::beginTransaction();
            $user = User::create([
                'name' => $request['name'],
                'instansi_id' => $request['instansi_id'],
                'username' => $request['email'],
                'email' => $request['email'],
                'password' => Hash::make($request['password']),
                'menuroles' => $role->name,
                'phone' => '0'
            ]);

            $user->assignRole($role->name);
            DB::commit();
            return redirect()->route('users.index')->with('success', 'Successfully add user : '.$user->name);
        } catch (\Throwable $th) {
            DB::rollback();
            return redirect()->back()->with('fail', $th->getMessage())->withInput();
        }
    }
}
